from .scrap_teams import scrap_teams_by_year
from .scrap_race import scrap_stage, scrap_stage_ttt, scrap_stage_gc, scrap_final_gc, scrap_final_points, \
    scrap_final_kom, scrap_final_youth, scrap_final_teams



from pyvirtualdisplay import Display
from selenium import webdriver
import os


def init_driver() -> webdriver:
    if 'MAC' not in os.environ:
        print("Initializing display")
        display = Display(visible=False, size=(1200, 1200))
        display.start()
        print("Display initialized")

    # Create a new Chrome session
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--disable-dev-shm-usage')
    prefs = {"profile.managed_default_content_settings.images": 2}
    chrome_options.add_experimental_option("prefs", prefs)
    print("Initializing Chromedriver")
    driver = webdriver.Chrome(options=chrome_options)
    driver.implicitly_wait(30)
    driver.maximize_window()
    print("Chromedriver initialized")

    return driver



from selenium import webdriver
from selenium.webdriver.common.by import By

from .base import init_driver

BASE_URL: str = "https://www.procyclingstats.com/race/{args}"
STAGE_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-{stage}")
STAGE_GC_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-{stage}-gc")
FINAL_GC_URL: str = BASE_URL.format(args="{race_name}/{year}/gc")
FINAL_POINTS_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-21-points")
FINAL_KOM_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-21-kom")
FINAL_YOUTH_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-21-youth")
FINAL_TEAMS_URL: str = BASE_URL.format(args="{race_name}/{year}/stage-21-teams")

STAGE_LEN = 20
STAGE_TTT_LEN = 3
STAGE_GC_LEN = 6
FINAL_GC_LEN = 25
FINAL_POINTS_LEN = 1
FINAL_KOM_LEN = 1
FINAL_YOUTH_LEN = 1
FINAL_TEAMS_LEN = 1


def scrap_items(url: str, results_len: int) -> list:
    driver: webdriver = init_driver()
    driver.get(url)
    result_items = driver.find_element(By.XPATH,
                                       "//div[contains(@class, 'result-cont') and not(contains(@class, 'hide'))]") \
        .find_element(By.XPATH, ".//table[contains(@class, 'basic') and contains(@class, 'results')]") \
        .find_element(By.XPATH, ".//tbody").find_elements(By.XPATH, ".//tr")
    results = list()
    for index in range(0, results_len):
        result_link = result_items[index].find_element(By.XPATH, ".//a")
        results.append({
            "name": result_link.get_attribute("innerHTML"),
            "link": result_link.get_attribute("href"),
            "position": index + 1
        })
    # Close the driver
    driver.close()
    print("Results scraped. Webdriver closed!")
    return results


def scrap_items_ttt(url: str, team_results_len: int) -> list:
    driver: webdriver = init_driver()
    driver.get(url)
    result_items = driver.find_element(By.XPATH,
                                       "//div[contains(@class, 'result-cont') and not(contains(@class, 'hide'))]") \
        .find_element(By.XPATH, ".//table[contains(@class, 'results-ttt')]") \
        .find_element(By.XPATH, ".//tbody").find_elements(By.XPATH, ".//tr")
    results = list()
    team_position = 0
    for item in result_items:
        if 'team' in item.get_attribute('class').split():
            if team_position >= team_results_len:
                break
            else:
                team_position += 1
                pass
        result_element = item.find_element(By.XPATH, ".//a")
        results.append({
            "name": result_element.get_attribute("innerHTML"),
            "link": result_element.get_attribute("href"),
            "position": team_position
        })
    # Close the driver
    driver.close()
    print("Results scraped. Webdriver closed!")
    return results


def scrap_stage(race_name: str, year: int, stage: int, length: int = STAGE_LEN) -> list:
    stage_url = STAGE_URL.format(race_name=race_name, year=year, stage=stage)
    return scrap_items(stage_url, length)


def scrap_stage_ttt(race_name: str, year: int, stage: int, length: int = STAGE_TTT_LEN) -> list:
    stage_url = STAGE_URL.format(race_name=race_name, year=year, stage=stage)
    return scrap_items_ttt(stage_url, length)


def scrap_stage_gc(race_name: str, year: int, stage: int, length: int = STAGE_GC_LEN) -> list:
    stage_gc_url = STAGE_GC_URL.format(race_name=race_name, year=year, stage=stage)
    return scrap_items(stage_gc_url, length)


def scrap_final_gc(race_name: str, year: int, length: int = FINAL_GC_LEN) -> list:
    final_gc_url = FINAL_GC_URL.format(race_name=race_name, year=year)
    return scrap_items(final_gc_url, length)


def scrap_final_points(race_name: str, year: int, length: int = FINAL_POINTS_LEN) -> list:
    final_points_url = FINAL_POINTS_URL.format(race_name=race_name, year=year)
    return scrap_items(final_points_url, length)


def scrap_final_kom(race_name: str, year: int, length: int = FINAL_KOM_LEN) -> list:
    final_kom_url = FINAL_KOM_URL.format(race_name=race_name, year=year)
    return scrap_items(final_kom_url, length)


def scrap_final_youth(race_name: str, year: int, length: int = FINAL_YOUTH_LEN) -> list:
    final_youth_url = FINAL_YOUTH_URL.format(race_name=race_name, year=year)
    return scrap_items(final_youth_url, length)


def scrap_final_teams(race_name: str, year: int, length: int = FINAL_TEAMS_LEN) -> list:
    final_teams_url = FINAL_TEAMS_URL.format(race_name=race_name, year=year)
    return scrap_items(final_teams_url, length)



from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException

from .base import init_driver
from selenium import webdriver

BASE_URL: str = "https://www.procyclingstats.com/teams.php?{args}"
TEAMS_BY_YEAR_URL: str = BASE_URL.format(args="year={year}")


def scrap_teams_by_year(year: int) -> list:
    driver: webdriver = init_driver()

    url = TEAMS_BY_YEAR_URL.format(year=year)
    driver.get(url)

    elements = driver.find_elements(By.XPATH, "//ul[contains(@class, 'list') and contains(@class, 'fs14') and "
                                              "contains(@class, 'columns2') and contains(@class, 'mob_columns1')]")

    # Get the teams from each category
    teams = list()
    for uci_category in elements:
        for team_element in uci_category.find_elements(By.XPATH, ".//li"):
            team = dict()
            link = team_element.find_element(By.XPATH, ".//a")
            team['link'] = link.get_attribute('href')
            team['name'] = link.text
            team['riders'] = list()
            teams.append(team)

    # Get the img and the riders from each team
    for team in teams:
        driver.get(team['link'])
        team_img_candidates = driver.find_elements(By.XPATH, "//ul[contains(@class, 'infolist')]")
        if len(team_img_candidates) == 1:
            team['img'] = team_img_candidates[0].find_element(By.XPATH, ".//img").get_attribute('src')
        else:
            for infolist_element in team_img_candidates:
                if 'Shirt:' in infolist_element.text:
                    team['img'] = infolist_element.find_element(By.XPATH, ".//img").get_attribute('src')

        riders = driver.find_element(By.XPATH, "//ul[contains(@class, 'list') and contains(@class, 'pad2')]") \
            .find_elements(By.XPATH, ".//li")
        for rider_element in riders:
            rider = dict()
            rider_link = rider_element.find_element(By.XPATH, ".//a")
            rider['link'] = rider_link.get_attribute('href')
            rider['preferred_name'] = rider_link.get_attribute('innerHTML')
            team['riders'].append(rider)

    # Get the information of each rider
    for team in teams:
        for rider in team['riders']:
            driver.get(rider['link'])
            try:
                rider['img'] = driver.find_element(By.XPATH, "//div[contains(@class, 'rdr-img-cont')]") \
                    .find_element(By.XPATH, ".//img").get_attribute('src')
            except NoSuchElementException:
                print("Could not find img for '" + rider["preferred_name"] + "' in '" + rider["link"] + "'")
            rider['country'] = driver.find_element(By.XPATH, "//div[contains(@class, 'rdr-info-cont')]") \
                .find_element(By.XPATH, ".//a[contains(@class, 'black')]").get_attribute('innerHTML')

    # Close the driver
    driver.close()

    return teams
